from odoo import models, fields
from odoo.exceptions import AccessError


class RestrictedAllowed():
    SEO_FIELDS = set([
        'website_meta_title', 'website_meta_description',
        'website_meta_keywords', 'website_meta_og_img',
        'seo_name',
    ])
    ARCH_FIELDS = set(['arch', 'arch_db'])
    pass


class Views(models.Model):
    _inherit = 'ir.ui.view'

    def write(self, vals):
        if isinstance(self.env.context.get('restricted_sudo'), RestrictedAllowed):
            # if we write only on whitelisted fields for restricted editor
            if not (set(vals.keys()) - RestrictedAllowed.SEO_FIELDS - RestrictedAllowed.ARCH_FIELDS):
                # a last check, in theory the context cannot have this keys if not restricted editor
                # but if smbdy find a way to set an object in a rpc call or safe_eval... we check
                # that it is at least a employee with minimal privileges... Just an extra mile.
                if self.env.user.has_group('website.group_website_restricted_editor'):
                    self = self.sudo()

        super().write(vals)

    def save(self, value, xpath=None):
        self.ensure_one()
        if (
            xpath
            and self.page_ids._check_restricted_can_edit()
        ):
            self = self.with_context(restricted_sudo=RestrictedAllowed())
        super().save(value, xpath=xpath)

    def _update_field_translations(self, fname, translations, digest=None):
        self.ensure_one()
        if fname == 'arch_db' and self.page_ids._check_restricted_can_edit():
            self = self.sudo()
        return super(Views, self)._update_field_translations(fname, translations, digest)


class Pages(models.Model):
    _inherit = 'website.page'

    page_group_ids = fields.Many2many('website.page.group')

    def write(self, vals):
        if (
            self._check_restricted_can_edit()
            and not set(vals.keys()) - RestrictedAllowed.SEO_FIELDS  # and "only SEO_FIELDS"
        ):
            # write on page.website_meta_* field, will write on ir.ui.view field due to the inherits
            # so we need to add a sudo() because we only give right to website.page
            self = self.with_context(restricted_sudo=RestrictedAllowed())
        super().write(vals)

    def _check_restricted_can_edit(self):
        if not self:  # could not see them due to ir.rule in case of view_id.page_ids e.g.
            return False
        if (
            not self.env.user.has_group('website.group_website_designer') and
            self.env.user.has_group('website.group_website_restricted_editor')
        ):
            for page in self:
                if not page.page_group_ids:
                    return False
                try:
                    page.sudo(False).check_access_rights('write')
                    page.sudo(False).check_access_rule('write')
                except AccessError:
                    return False
            return True
        return False


class PageGroup(models.Model):
    _name = 'website.page.group'
    _description = "Website page group"

    name = fields.Char()
    description = fields.Text()
    page_ids = fields.Many2many('website.page')
    user_ids = fields.Many2many('res.users', domain="[('share', '=', False)]")


class IrQWeb(models.AbstractModel):
    _inherit = 'ir.qweb'

    def _prepare_frontend_environment(self, values):
        irQweb = super()._prepare_frontend_environment(values)
        # if restricted, need to check if user need branding for this page
        if irQweb.env.context.get('inherit_branding_auto'):
            if values.get('main_object'):
                if values['main_object']._name == 'website.page':
                    page = values['main_object']
                    if page._check_restricted_can_edit():
                        irQweb = irQweb.with_context(inherit_branding=True)
                else:
                    view = False
                    if values['main_object']._name == 'ir.ui.view':
                        view = values['main_object']
                    elif values.get('seo_object') and values['seo_object']._name == 'ir.ui.view':
                        view = values['seo_object']
                    if view and view.page_ids._check_restricted_can_edit():
                        irQweb = irQweb.with_context(inherit_branding=True)
        return irQweb
